@@ -124,23 +124,27 @@ class PHPUnit_Framework_MockObject_Generator
     /**
      * Returns a mock object for the specified class.
      *
-     * @param  array|string                                  $type
-     * @param  array                                         $methods
-     * @param  array                                         $arguments
-     * @param  string                                        $mockClassName
-     * @param  bool                                          $callOriginalConstructor
-     * @param  bool                                          $callOriginalClone
-     * @param  bool                                          $callAutoload
-     * @param  bool                                          $cloneArguments
-     * @param  bool                                          $callOriginalMethods
-     * @param  object                                        $proxyTarget
-     * @return object
+     * @param array|string $type
+     * @param array        $methods
+     * @param array        $arguments
+     * @param string       $mockClassName
+     * @param bool         $callOriginalConstructor
+     * @param bool         $callOriginalClone
+     * @param bool         $callAutoload
+     * @param bool         $cloneArguments
+     * @param bool         $callOriginalMethods
+     * @param object       $proxyTarget
+     * @param bool         $allowMockingUnknownTypes
+     *
+     * @return PHPUnit_Framework_MockObject_MockObject
+     *
      * @throws InvalidArgumentException
      * @throws PHPUnit_Framework_Exception
      * @throws PHPUnit_Framework_MockObject_RuntimeException
+     *
      * @since  Method available since Release 1.0.0
      */
-    public function getMock($type, $methods = [], array $arguments = [], $mockClassName = '', $callOriginalConstructor = true, $callOriginalClone = true, $callAutoload = true, $cloneArguments = true, $callOriginalMethods = false, $proxyTarget = null)
+    public function getMock($type, $methods = [], array $arguments = [], $mockClassName = '', $callOriginalConstructor = true, $callOriginalClone = true, $callAutoload = true, $cloneArguments = true, $callOriginalMethods = false, $proxyTarget = null, $allowMockingUnknownTypes = true)
     {
         if (!is_array($type) && !is_string($type)) {
             throw PHPUnit_Util_InvalidArgumentHelper::factory(1, 'array or string');
@@ -159,24 +163,53 @@ class PHPUnit_Framework_MockObject_Generator
         }
 
         if (is_array($type)) {
-            $type = array_unique(array_map(
-                function ($type) {
-                    if ($type === 'Traversable' ||
-                      $type === '\\Traversable' ||
-                      $type === '\\Iterator') {
-                        return 'Iterator';
-                    }
+            $type = array_unique(
+                array_map(
+                    function ($type) {
+                        if ($type === 'Traversable' ||
+                            $type === '\\Traversable' ||
+                            $type === '\\Iterator') {
+                            return 'Iterator';
+                        }
 
-                    return $type;
-                },
-                $type
-            ));
+                        return $type;
+                    },
+                    $type
+                )
+            );
+        }
+
+        if (!$allowMockingUnknownTypes) {
+            if (is_array($type)) {
+                foreach ($type as $_type) {
+                    if (!class_exists($_type, $callAutoload) &&
+                        !interface_exists($_type, $callAutoload)) {
+                        throw new PHPUnit_Framework_MockObject_RuntimeException(
+                            sprintf(
+                                'Cannot stub or mock class or interface "%s" which does not exist',
+                                $_type
+                            )
+                        );
+                    }
+                }
+            } else {
+                if (!class_exists($type, $callAutoload) &&
+                    !interface_exists($type, $callAutoload)
+                ) {
+                    throw new PHPUnit_Framework_MockObject_RuntimeException(
+                        sprintf(
+                            'Cannot stub or mock class or interface "%s" which does not exist',
+                            $type
+                        )
+                    );
+                }
+            }
         }
 
         if (null !== $methods) {
             foreach ($methods as $method) {
                 if (!preg_match('~[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*~', $method)) {
-                    throw new PHPUnit_Framework_Exception(
+                    throw new PHPUnit_Framework_MockObject_RuntimeException(
                         sprintf(
                             'Cannot stub or mock method with invalid name "%s"',
                             $method
@@ -188,8 +221,9 @@ class PHPUnit_Framework_MockObject_Generator
             if ($methods != array_unique($methods)) {
                 throw new PHPUnit_Framework_MockObject_RuntimeException(
                     sprintf(
-                        'Cannot stub or mock using a method list that contains duplicates: "%s"',
-                        implode(', ', $methods)
+                        'Cannot stub or mock using a method list that contains duplicates: "%s" (duplicate: "%s")',
+                        implode(', ', $methods),
+                        implode(', ', array_unique(array_diff_assoc($methods, array_unique($methods))))
                     )
                 );
             }
@@ -237,14 +271,15 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  string       $code
-     * @param  string       $className
-     * @param  array|string $type
-     * @param  bool         $callOriginalConstructor
-     * @param  bool         $callAutoload
-     * @param  array        $arguments
-     * @param  bool         $callOriginalMethods
-     * @param  object       $proxyTarget
+     * @param string       $code
+     * @param string       $className
+     * @param array|string $type
+     * @param bool         $callOriginalConstructor
+     * @param bool         $callAutoload
+     * @param array        $arguments
+     * @param bool         $callOriginalMethods
+     * @param object       $proxyTarget
+     *
      * @return object
      */
     private function getObject($code, $className, $type = '', $callOriginalConstructor = false, $callAutoload = false, array $arguments = [], $callOriginalMethods = false, $proxyTarget = null)
@@ -311,16 +346,19 @@ class PHPUnit_Framework_MockObject_Generator
      * methods of the class mocked. Concrete methods to mock can be specified with
      * the last parameter
      *
-     * @param  string $originalClassName
-     * @param  array  $arguments
-     * @param  string $mockClassName
-     * @param  bool   $callOriginalConstructor
-     * @param  bool   $callOriginalClone
-     * @param  bool   $callAutoload
-     * @param  array  $mockedMethods
-     * @param  bool   $cloneArguments
-     * @return object
+     * @param string $originalClassName
+     * @param array  $arguments
+     * @param string $mockClassName
+     * @param bool   $callOriginalConstructor
+     * @param bool   $callOriginalClone
+     * @param bool   $callAutoload
+     * @param array  $mockedMethods
+     * @param bool   $cloneArguments
+     *
+     * @return PHPUnit_Framework_MockObject_MockObject
+     *
      * @since  Method available since Release 1.0.0
+     *
      * @throws PHPUnit_Framework_MockObject_RuntimeException
      * @throws PHPUnit_Framework_Exception
      */
@@ -371,16 +409,19 @@ class PHPUnit_Framework_MockObject_Generator
      * of the trait mocked. Concrete methods to mock can be specified with the
      * `$mockedMethods` parameter.
      *
-     * @param  string $traitName
-     * @param  array  $arguments
-     * @param  string $mockClassName
-     * @param  bool   $callOriginalConstructor
-     * @param  bool   $callOriginalClone
-     * @param  bool   $callAutoload
-     * @param  array  $mockedMethods
-     * @param  bool   $cloneArguments
-     * @return object
+     * @param string $traitName
+     * @param array  $arguments
+     * @param string $mockClassName
+     * @param bool   $callOriginalConstructor
+     * @param bool   $callOriginalClone
+     * @param bool   $callAutoload
+     * @param array  $mockedMethods
+     * @param bool   $cloneArguments
+     *
+     * @return PHPUnit_Framework_MockObject_MockObject
+     *
      * @since  Method available since Release 1.2.3
+     *
      * @throws PHPUnit_Framework_MockObject_RuntimeException
      * @throws PHPUnit_Framework_Exception
      */
@@ -417,9 +458,9 @@ class PHPUnit_Framework_MockObject_Generator
 
         $classTemplate->setVar(
             [
-            'prologue'   => 'abstract ',
-            'class_name' => $className['className'],
-            'trait_name' => $traitName
+                'prologue'   => 'abstract ',
+                'class_name' => $className['className'],
+                'trait_name' => $traitName
             ]
         );
 
@@ -434,14 +475,17 @@ class PHPUnit_Framework_MockObject_Generator
     /**
      * Returns an object for the specified trait.
      *
-     * @param  string $traitName
-     * @param  array  $arguments
-     * @param  string $traitClassName
-     * @param  bool   $callOriginalConstructor
-     * @param  bool   $callOriginalClone
-     * @param  bool   $callAutoload
+     * @param string $traitName
+     * @param array  $arguments
+     * @param string $traitClassName
+     * @param bool   $callOriginalConstructor
+     * @param bool   $callOriginalClone
+     * @param bool   $callAutoload
+     *
      * @return object
+     *
      * @since  Method available since Release 1.1.0
+     *
      * @throws PHPUnit_Framework_MockObject_RuntimeException
      * @throws PHPUnit_Framework_Exception
      */
@@ -478,9 +522,9 @@ class PHPUnit_Framework_MockObject_Generator
 
         $classTemplate->setVar(
             [
-            'prologue'   => '',
-            'class_name' => $className['className'],
-            'trait_name' => $traitName
+                'prologue'   => '',
+                'class_name' => $className['className'],
+                'trait_name' => $traitName
             ]
         );
 
@@ -491,13 +535,14 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  array|string $type
-     * @param  array        $methods
-     * @param  string       $mockClassName
-     * @param  bool         $callOriginalClone
-     * @param  bool         $callAutoload
-     * @param  bool         $cloneArguments
-     * @param  bool         $callOriginalMethods
+     * @param array|string $type
+     * @param array        $methods
+     * @param string       $mockClassName
+     * @param bool         $callOriginalClone
+     * @param bool         $callAutoload
+     * @param bool         $cloneArguments
+     * @param bool         $callOriginalMethods
+     *
      * @return array
      */
     public function generate($type, array $methods = null, $mockClassName = '', $callOriginalClone = true, $callAutoload = true, $cloneArguments = true, $callOriginalMethods = false)
@@ -538,11 +583,13 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  string                                        $wsdlFile
-     * @param  string                                        $className
-     * @param  array                                         $methods
-     * @param  array                                         $options
+     * @param string $wsdlFile
+     * @param string $className
+     * @param array  $methods
+     * @param array  $options
+     *
      * @return string
+     *
      * @throws PHPUnit_Framework_MockObject_RuntimeException
      */
     public function generateClassFromWsdl($wsdlFile, $className, array $methods = [], array $options = [])
@@ -626,15 +673,17 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  array|string                $type
-     * @param  array|null                  $methods
-     * @param  string                      $mockClassName
-     * @param  bool                        $callOriginalClone
-     * @param  bool                        $callAutoload
-     * @param  bool                        $cloneArguments
-     * @param  bool                        $callOriginalMethods
+     * @param array|string $type
+     * @param array|null   $methods
+     * @param string       $mockClassName
+     * @param bool         $callOriginalClone
+     * @param bool         $callAutoload
+     * @param bool         $cloneArguments
+     * @param bool         $callOriginalMethods
+     *
      * @return array
-     * @throws PHPUnit_Framework_Exception
+     *
+     * @throws PHPUnit_Framework_MockObject_RuntimeException
      */
     private function generateMock($type, $methods, $mockClassName, $callOriginalClone, $callAutoload, $cloneArguments, $callOriginalMethods)
     {
@@ -658,7 +707,7 @@ class PHPUnit_Framework_MockObject_Generator
         if (is_array($type)) {
             foreach ($type as $_type) {
                 if (!interface_exists($_type, $callAutoload)) {
-                    throw new PHPUnit_Framework_Exception(
+                    throw new PHPUnit_Framework_MockObject_RuntimeException(
                         sprintf(
                             'Interface "%s" does not exist.',
                             $_type
@@ -670,7 +719,7 @@ class PHPUnit_Framework_MockObject_Generator
 
                 foreach ($this->getClassMethods($_type) as $method) {
                     if (in_array($method, $methods)) {
-                        throw new PHPUnit_Framework_Exception(
+                        throw new PHPUnit_Framework_MockObject_RuntimeException(
                             sprintf(
                                 'Duplicate method "%s" not allowed.',
                                 $method
@@ -685,14 +734,11 @@ class PHPUnit_Framework_MockObject_Generator
 
         if (class_exists($mockClassName['fullClassName'], $callAutoload)) {
             $isClass = true;
-        } else {
-            if (interface_exists($mockClassName['fullClassName'], $callAutoload)) {
-                $isInterface = true;
-            }
+        } elseif (interface_exists($mockClassName['fullClassName'], $callAutoload)) {
+            $isInterface = true;
         }
 
-        if (!class_exists($mockClassName['fullClassName'], $callAutoload) &&
-            !interface_exists($mockClassName['fullClassName'], $callAutoload)) {
+        if (!$isClass && !$isInterface) {
             $prologue = 'class ' . $mockClassName['originalClassName'] . "\n{\n}\n\n";
 
             if (!empty($mockClassName['namespaceName'])) {
@@ -710,7 +756,7 @@ class PHPUnit_Framework_MockObject_Generator
             $class = new ReflectionClass($mockClassName['fullClassName']);
 
             if ($class->isFinal()) {
-                throw new PHPUnit_Framework_Exception(
+                throw new PHPUnit_Framework_MockObject_RuntimeException(
                     sprintf(
                         'Class "%s" is declared "final" and cannot be mocked.',
                         $mockClassName['fullClassName']
@@ -753,6 +799,13 @@ class PHPUnit_Framework_MockObject_Generator
         }
 
         $mockedMethods = '';
+        $configurable  = [];
+
+        foreach ($methods as $methodName) {
+            if ($methodName != '__construct' && $methodName != '__clone') {
+                $configurable[] = strtolower($methodName);
+            }
+        }
 
         if (isset($class)) {
             // https://github.com/sebastianbergmann/phpunit-mock-objects/issues/103
@@ -797,7 +850,7 @@ class PHPUnit_Framework_MockObject_Generator
 
         $method = '';
 
-        if (!in_array('method', $methods)) {
+        if (!in_array('method', $methods) && (!isset($class) || !$class->hasMethod('method'))) {
             $methodTemplate = new Text_Template(
                 $templateDir . 'mocked_class_method.tpl'
             );
@@ -807,17 +860,18 @@ class PHPUnit_Framework_MockObject_Generator
 
         $classTemplate->setVar(
             [
-            'prologue'          => isset($prologue) ? $prologue : '',
-            'epilogue'          => isset($epilogue) ? $epilogue : '',
-            'class_declaration' => $this->generateMockClassDeclaration(
-                $mockClassName,
-                $isInterface,
-                $additionalInterfaces
-            ),
-            'clone'             => $cloneTemplate,
-            'mock_class_name'   => $mockClassName['className'],
-            'mocked_methods'    => $mockedMethods,
-            'method'            => $method
+                'prologue'          => isset($prologue) ? $prologue : '',
+                'epilogue'          => isset($epilogue) ? $epilogue : '',
+                'class_declaration' => $this->generateMockClassDeclaration(
+                    $mockClassName,
+                    $isInterface,
+                    $additionalInterfaces
+                ),
+                'clone'             => $cloneTemplate,
+                'mock_class_name'   => $mockClassName['className'],
+                'mocked_methods'    => $mockedMethods,
+                'method'            => $method,
+                'configurable'      => '[' . implode(', ', array_map(function ($m) { return '\'' . $m . '\'';}, $configurable)) . ']'
             ]
         );
 
@@ -828,9 +882,10 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  array|string $type
-     * @param  string       $className
-     * @param  string       $prefix
+     * @param array|string $type
+     * @param string       $className
+     * @param string       $prefix
+     *
      * @return array
      */
     private function generateClassName($type, $className, $prefix)
@@ -870,9 +925,10 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  array $mockClassName
-     * @param  bool  $isInterface
-     * @param  array $additionalInterfaces
+     * @param array $mockClassName
+     * @param bool  $isInterface
+     * @param array $additionalInterfaces
+     *
      * @return array
      */
     private function generateMockClassDeclaration(array $mockClassName, $isInterface, array $additionalInterfaces = [])
@@ -912,10 +968,11 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  string           $templateDir
-     * @param  ReflectionMethod $method
-     * @param  bool             $cloneArguments
-     * @param  bool             $callOriginalMethods
+     * @param string           $templateDir
+     * @param ReflectionMethod $method
+     * @param bool             $cloneArguments
+     * @param bool             $callOriginalMethods
+     *
      * @return string
      */
     private function generateMockedMethodDefinitionFromExisting($templateDir, ReflectionMethod $method, $cloneArguments, $callOriginalMethods)
@@ -944,6 +1001,12 @@ class PHPUnit_Framework_MockObject_Generator
             $returnType = '';
         }
 
+        if (preg_match('#\*[ \t]*+@deprecated[ \t]*+(.*?)\r?+\n[ \t]*+\*(?:[ \t]*+@|/$)#s', $method->getDocComment(), $deprecation)) {
+            $deprecation = trim(preg_replace('#[ \t]*\r?\n[ \t]*+\*[ \t]*+#', ' ', $deprecation[1]));
+        } else {
+            $deprecation = false;
+        }
+
         return $this->generateMockedMethodDefinition(
             $templateDir,
             $method->getDeclaringClass()->getName(),
@@ -955,25 +1018,28 @@ class PHPUnit_Framework_MockObject_Generator
             $returnType,
             $reference,
             $callOriginalMethods,
-            $method->isStatic()
+            $method->isStatic(),
+            $deprecation
         );
     }
 
     /**
-     * @param  string $templateDir
-     * @param  string $className
-     * @param  string $methodName
-     * @param  bool   $cloneArguments
-     * @param  string $modifier
-     * @param  string $arguments_decl
-     * @param  string $arguments_call
-     * @param  string $return_type
-     * @param  string $reference
-     * @param  bool   $callOriginalMethods
-     * @param  bool   $static
+     * @param string $templateDir
+     * @param string $className
+     * @param string $methodName
+     * @param bool   $cloneArguments
+     * @param string $modifier
+     * @param string $arguments_decl
+     * @param string $arguments_call
+     * @param string $return_type
+     * @param string $reference
+     * @param bool   $callOriginalMethods
+     * @param bool   $static
+     * @param string $deprecation
+     *
      * @return string
      */
-    private function generateMockedMethodDefinition($templateDir, $className, $methodName, $cloneArguments = true, $modifier = 'public', $arguments_decl = '', $arguments_call = '', $return_type = '', $reference = '', $callOriginalMethods = false, $static = false)
+    private function generateMockedMethodDefinition($templateDir, $className, $methodName, $cloneArguments = true, $modifier = 'public', $arguments_decl = '', $arguments_call = '', $return_type = '', $reference = '', $callOriginalMethods = false, $static = false, $deprecation = false)
     {
         if ($static) {
             $templateFile = 'mocked_static_method.tpl';
@@ -991,20 +1057,32 @@ class PHPUnit_Framework_MockObject_Generator
             $return_type = $className;
         }
 
+        if (false !== $deprecation) {
+            $deprecation = "The $className::$methodName method is deprecated ($deprecation).";
+            $deprecationTemplate = new Text_Template($templateDir . 'deprecation.tpl');
+            $deprecationTemplate->setVar(
+                array(
+                'deprecation' => var_export($deprecation, true),
+                )
+            );
+            $deprecation = $deprecationTemplate->render();
+        }
+
         $template = new Text_Template($templateDir . $templateFile);
 
         $template->setVar(
             [
-            'arguments_decl'  => $arguments_decl,
-            'arguments_call'  => $arguments_call,
-            'return_delim'    => $return_type ? ': ' : '',
-            'return_type'     => $return_type,
-            'arguments_count' => !empty($arguments_call) ? count(explode(',', $arguments_call)) : 0,
-            'class_name'      => $className,
-            'method_name'     => $methodName,
-            'modifier'        => $modifier,
-            'reference'       => $reference,
-            'clone_arguments' => $cloneArguments ? 'true' : 'false'
+                'arguments_decl'  => $arguments_decl,
+                'arguments_call'  => $arguments_call,
+                'return_delim'    => $return_type ? ': ' : '',
+                'return_type'     => $return_type,
+                'arguments_count' => !empty($arguments_call) ? count(explode(',', $arguments_call)) : 0,
+                'class_name'      => $className,
+                'method_name'     => $methodName,
+                'modifier'        => $modifier,
+                'reference'       => $reference,
+                'clone_arguments' => $cloneArguments ? 'true' : 'false',
+                'deprecation'     => $deprecation
             ]
         );
 
@@ -1012,7 +1090,8 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  ReflectionMethod $method
+     * @param ReflectionMethod $method
+     *
      * @return bool
      */
     private function canMockMethod(ReflectionMethod $method)
@@ -1032,7 +1111,8 @@ class PHPUnit_Framework_MockObject_Generator
      *
      * Since PHP 7 the only names that are still reserved for method names are the ones that start with an underscore
      *
-     * @param  string $name
+     * @param string $name
+     *
      * @return bool
      */
     private function isMethodNameBlacklisted($name)
@@ -1051,10 +1131,13 @@ class PHPUnit_Framework_MockObject_Generator
     /**
      * Returns the parameters of a function or method.
      *
-     * @param  ReflectionMethod                              $method
-     * @param  bool                                          $forCall
+     * @param ReflectionMethod $method
+     * @param bool             $forCall
+     *
      * @return string
+     *
      * @throws PHPUnit_Framework_MockObject_RuntimeException
+     *
      * @since  Method available since Release 2.0.0
      */
     private function getMethodParameters(ReflectionMethod $method, $forCall = false)
@@ -1084,7 +1167,7 @@ class PHPUnit_Framework_MockObject_Generator
             $typeDeclaration = '';
 
             if (!$forCall) {
-                if ($this->hasType($parameter)) {
+                if ($this->hasType($parameter) && (string) $parameter->getType() !== 'self') {
                     $typeDeclaration = (string) $parameter->getType() . ' ';
                 } elseif ($parameter->isArray()) {
                     $typeDeclaration = 'array ';
@@ -1132,8 +1215,10 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  ReflectionParameter $parameter
+     * @param ReflectionParameter $parameter
+     *
      * @return bool
+     *
      * @since  Method available since Release 2.2.1
      */
     private function isVariadic(ReflectionParameter $parameter)
@@ -1142,8 +1227,10 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  ReflectionParameter $parameter
+     * @param ReflectionParameter $parameter
+     *
      * @return bool
+     *
      * @since  Method available since Release 2.3.4
      */
     private function hasType(ReflectionParameter $parameter)
@@ -1152,7 +1239,8 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  ReflectionMethod $method
+     * @param ReflectionMethod $method
+     *
      * @return bool
      */
     private function hasReturnType(ReflectionMethod $method)
@@ -1161,11 +1249,13 @@ class PHPUnit_Framework_MockObject_Generator
     }
 
     /**
-     * @param  string $className
+     * @param string $className
+     *
      * @return array
+     *
      * @since  Method available since Release 2.3.2
      */
-    private function getClassMethods($className)
+    public function getClassMethods($className)
     {
         $class   = new ReflectionClass($className);
         $methods = [];
