@@ -8,29 +8,32 @@
  * file that was distributed with this source code.
  */
 
-/**
- * @since      Class available since Release 3.0.0
- */
 class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 {
     public function testMockedMethodIsNeverCalled()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->never())
              ->method('doSomething');
     }
 
     public function testMockedMethodIsNeverCalledWithParameter()
     {
-        $mock = $this->getMock('SomeClass');
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->getMock();
+
         $mock->expects($this->never())
-            ->method('doSomething')
-            ->with('someArg');
+             ->method('doSomething')
+             ->with('someArg');
     }
 
     public function testMockedMethodIsNotCalledWhenExpectsAnyWithParameter()
     {
-        $mock = $this->getMock('SomeClass');
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomethingElse')
              ->with('someArg');
@@ -38,14 +41,18 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsNotCalledWhenMethodSpecifiedDirectlyWithParameter()
     {
-        $mock = $this->getMock('SomeClass');
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->getMock();
+
         $mock->method('doSomethingElse')
-            ->with('someArg');
+             ->with('someArg');
     }
 
     public function testMockedMethodIsCalledAtLeastOnce()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->atLeastOnce())
              ->method('doSomething');
 
@@ -54,7 +61,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledAtLeastOnce2()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->atLeastOnce())
              ->method('doSomething');
 
@@ -64,7 +73,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledAtLeastTwice()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->atLeast(2))
              ->method('doSomething');
 
@@ -74,7 +85,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledAtLeastTwice2()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->atLeast(2))
              ->method('doSomething');
 
@@ -85,7 +98,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledAtMostTwice()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->atMost(2))
              ->method('doSomething');
 
@@ -95,7 +110,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledAtMosttTwice2()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->atMost(2))
              ->method('doSomething');
 
@@ -104,7 +121,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledOnce()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('doSomething');
 
@@ -113,7 +132,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledOnceWithParameter()
     {
-        $mock = $this->getMock('SomeClass');
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('doSomethingElse')
              ->with($this->equalTo('something'));
@@ -123,7 +144,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockedMethodIsCalledExactly()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->exactly(2))
              ->method('doSomething');
 
@@ -133,7 +156,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStubbedException()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->will($this->throwException(new Exception));
@@ -149,7 +174,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStubbedWillThrowException()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->willThrowException(new Exception);
@@ -165,14 +192,18 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStubbedReturnValue()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->will($this->returnValue('something'));
 
         $this->assertEquals('something', $mock->doSomething());
 
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->willReturn('something');
@@ -187,7 +218,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
             ['e', 'f', 'g', 'h']
         ];
 
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->will($this->returnValueMap($map));
@@ -196,7 +229,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('h', $mock->doSomething('e', 'f', 'g'));
         $this->assertEquals(null, $mock->doSomething('foo', 'bar'));
 
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->willReturnMap($map);
@@ -208,14 +243,18 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStubbedReturnArgument()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->will($this->returnArgument(1));
 
         $this->assertEquals('b', $mock->doSomething('a', 'b'));
 
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->willReturnArgument(1);
@@ -225,14 +264,20 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testFunctionCallback()
     {
-        $mock = $this->getMock('SomeClass', ['doSomething'], [], '', false);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['doSomething'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('doSomething')
              ->will($this->returnCallback('functionCallback'));
 
         $this->assertEquals('pass', $mock->doSomething('foo', 'bar'));
 
-        $mock = $this->getMock('SomeClass', ['doSomething'], [], '', false);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['doSomething'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('doSomething')
              ->willReturnCallback('functionCallback');
@@ -242,14 +287,18 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStubbedReturnSelf()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->will($this->returnSelf());
 
         $this->assertEquals($mock, $mock->doSomething());
 
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->willReturnSelf();
@@ -259,7 +308,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStubbedReturnOnConsecutiveCalls()
     {
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->will($this->onConsecutiveCalls('a', 'b', 'c'));
@@ -268,7 +319,9 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('b', $mock->doSomething());
         $this->assertEquals('c', $mock->doSomething());
 
-        $mock = $this->getMock('AnInterface');
+        $mock = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
         $mock->expects($this->any())
              ->method('doSomething')
              ->willReturnOnConsecutiveCalls('a', 'b', 'c');
@@ -280,7 +333,10 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStaticMethodCallback()
     {
-        $mock = $this->getMock('SomeClass', ['doSomething'], [], '', false);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['doSomething'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('doSomething')
              ->will($this->returnCallback(['MethodCallback', 'staticCallback']));
@@ -290,7 +346,10 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testPublicMethodCallback()
     {
-        $mock = $this->getMock('SomeClass', ['doSomething'], [], '', false);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['doSomething'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('doSomething')
              ->will($this->returnCallback([new MethodCallback, 'nonStaticCallback']));
@@ -300,19 +359,35 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockClassOnlyGeneratedOnce()
     {
-        $mock1 = $this->getMock('AnInterface');
-        $mock2 = $this->getMock('AnInterface');
+        $mock1 = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
+
+        $mock2 = $this->getMockBuilder(AnInterface::class)
+                     ->getMock();
 
         $this->assertEquals(get_class($mock1), get_class($mock2));
     }
 
     public function testMockClassDifferentForPartialMocks()
     {
-        $mock1 = $this->getMock('PartialMockTestClass');
-        $mock2 = $this->getMock('PartialMockTestClass', ['doSomething']);
-        $mock3 = $this->getMock('PartialMockTestClass', ['doSomething']);
-        $mock4 = $this->getMock('PartialMockTestClass', ['doAnotherThing']);
-        $mock5 = $this->getMock('PartialMockTestClass', ['doAnotherThing']);
+        $mock1 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->getMock();
+
+        $mock2 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMethods(['doSomething'])
+                      ->getMock();
+
+        $mock3 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMethods(['doSomething'])
+                      ->getMock();
+
+        $mock4 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMethods(['doAnotherThing'])
+                      ->getMock();
+
+        $mock5 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMethods(['doAnotherThing'])
+                      ->getMock();
 
         $this->assertNotEquals(get_class($mock1), get_class($mock2));
         $this->assertNotEquals(get_class($mock1), get_class($mock3));
@@ -326,11 +401,24 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testMockClassStoreOverrulable()
     {
-        $mock1 = $this->getMock('PartialMockTestClass');
-        $mock2 = $this->getMock('PartialMockTestClass', [], [], 'MyMockClassNameForPartialMockTestClass1');
-        $mock3 = $this->getMock('PartialMockTestClass');
-        $mock4 = $this->getMock('PartialMockTestClass', ['doSomething'], [], 'AnotherMockClassNameForPartialMockTestClass');
-        $mock5 = $this->getMock('PartialMockTestClass', [], [], 'MyMockClassNameForPartialMockTestClass2');
+        $mock1 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->getMock();
+
+        $mock2 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMockClassName('MyMockClassNameForPartialMockTestClass1')
+                      ->getMock();
+
+        $mock3 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->getMock();
+
+        $mock4 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMethods(['doSomething'])
+                      ->setMockClassName('AnotherMockClassNameForPartialMockTestClass')
+                      ->getMock();
+
+        $mock5 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->setMockClassName('MyMockClassNameForPartialMockTestClass2')
+                      ->getMock();
 
         $this->assertNotEquals(get_class($mock1), get_class($mock2));
         $this->assertEquals(get_class($mock1), get_class($mock3));
@@ -348,15 +436,19 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
      */
     public function testGetMockWithFixedClassNameCanProduceTheSameMockTwice()
     {
-        $mock = $this->getMockBuilder('StdClass')->setMockClassName('FixedName')->getMock();
-        $mock = $this->getMockBuilder('StdClass')->setMockClassName('FixedName')->getMock();
-        $this->assertInstanceOf('StdClass', $mock);
+        $mock = $this->getMockBuilder(stdClass::class)->setMockClassName('FixedName')->getMock();
+        $mock = $this->getMockBuilder(stdClass::class)->setMockClassName('FixedName')->getMock();
+        $this->assertInstanceOf(stdClass::class, $mock);
     }
 
     public function testOriginalConstructorSettingConsidered()
     {
-        $mock1 = $this->getMock('PartialMockTestClass');
-        $mock2 = $this->getMock('PartialMockTestClass', [], [], '', false);
+        $mock1 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->getMock();
+
+        $mock2 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->disableOriginalConstructor()
+                      ->getMock();
 
         $this->assertTrue($mock1->constructorCalled);
         $this->assertFalse($mock2->constructorCalled);
@@ -364,102 +456,129 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testOriginalCloneSettingConsidered()
     {
-        $mock1 = $this->getMock('PartialMockTestClass');
-        $mock2 = $this->getMock('PartialMockTestClass', [], [], '', true, false);
+        $mock1 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->getMock();
+
+        $mock2 = $this->getMockBuilder(PartialMockTestClass::class)
+                      ->disableOriginalClone()
+                      ->getMock();
 
         $this->assertNotEquals(get_class($mock1), get_class($mock2));
     }
 
     public function testGetMockForAbstractClass()
     {
-        $mock = $this->getMock('AbstractMockTestClass');
+        $mock = $this->getMockBuilder(AbstractMockTestClass::class)
+                     ->getMock();
+
         $mock->expects($this->never())
              ->method('doSomething');
     }
 
-    public function traversableProvider()
-    {
-        return [
-          ['Traversable'],
-          ['\Traversable'],
-          ['TraversableMockTestInterface'],
-          [['Traversable']],
-          [['Iterator','Traversable']],
-          [['\Iterator','\Traversable']]
-        ];
-    }
-
     /**
      * @dataProvider traversableProvider
      */
     public function testGetMockForTraversable($type)
     {
-        $mock = $this->getMock($type);
-        $this->assertInstanceOf('Traversable', $mock);
+        $mock = $this->getMockBuilder($type)
+                     ->getMock();
+
+        $this->assertInstanceOf(Traversable::class, $mock);
     }
 
     public function testMultipleInterfacesCanBeMockedInSingleObject()
     {
-        $mock = $this->getMock(['AnInterface', 'AnotherInterface']);
-        $this->assertInstanceOf('AnInterface', $mock);
-        $this->assertInstanceOf('AnotherInterface', $mock);
+        $mock = $this->getMockBuilder([AnInterface::class, AnotherInterface::class])
+                     ->getMock();
+
+        $this->assertInstanceOf(AnInterface::class, $mock);
+        $this->assertInstanceOf(AnotherInterface::class, $mock);
     }
 
     public function testGetMockForTrait()
     {
-        $mock = $this->getMockForTrait('AbstractTrait');
-        $mock->expects($this->never())->method('doSomething');
+        $mock = $this->getMockForTrait(AbstractTrait::class);
+
+        $mock->expects($this->never())
+             ->method('doSomething');
 
         $parent = get_parent_class($mock);
         $traits = class_uses($parent, false);
 
-        $this->assertContains('AbstractTrait', $traits);
+        $this->assertContains(AbstractTrait::class, $traits);
     }
 
     public function testClonedMockObjectShouldStillEqualTheOriginal()
     {
-        $a = $this->getMock('stdClass');
+        $a = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
         $b = clone $a;
+
         $this->assertEquals($a, $b);
     }
 
     public function testMockObjectsConstructedIndepentantlyShouldBeEqual()
     {
-        $a = $this->getMock('stdClass');
-        $b = $this->getMock('stdClass');
+        $a = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
+        $b = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
         $this->assertEquals($a, $b);
     }
 
     public function testMockObjectsConstructedIndepentantlyShouldNotBeTheSame()
     {
-        $a = $this->getMock('stdClass');
-        $b = $this->getMock('stdClass');
+        $a = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
+        $b = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
         $this->assertNotSame($a, $b);
     }
 
     public function testClonedMockObjectCanBeUsedInPlaceOfOriginalOne()
     {
-        $x = $this->getMock('stdClass');
+        $x = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
         $y = clone $x;
 
-        $mock = $this->getMock('stdClass', ['foo']);
-        $mock->expects($this->once())->method('foo')->with($this->equalTo($x));
+        $mock = $this->getMockBuilder(stdClass::class)
+                     ->setMethods(['foo'])
+                     ->getMock();
+
+        $mock->expects($this->once())
+             ->method('foo')
+             ->with($this->equalTo($x));
+
         $mock->foo($y);
     }
 
     public function testClonedMockObjectIsNotIdenticalToOriginalOne()
     {
-        $x = $this->getMock('stdClass');
+        $x = $this->getMockBuilder(stdClass::class)
+                  ->getMock();
+
         $y = clone $x;
 
-        $mock = $this->getMock('stdClass', ['foo']);
-        $mock->expects($this->once())->method('foo')->with($this->logicalNot($this->identicalTo($x)));
+        $mock = $this->getMockBuilder(stdClass::class)
+                     ->setMethods(['foo'])
+                     ->getMock();
+
+        $mock->expects($this->once())
+             ->method('foo')
+             ->with($this->logicalNot($this->identicalTo($x)));
+
         $mock->foo($y);
     }
 
     public function testObjectMethodCallWithArgumentCloningEnabled()
     {
-        $expectedObject = new StdClass;
+        $expectedObject = new stdClass;
 
         $mock = $this->getMockBuilder('SomeClass')
                      ->setMethods(['doSomethingElse'])
@@ -469,10 +588,14 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
         $actualArguments = [];
 
         $mock->expects($this->any())
-        ->method('doSomethingElse')
-        ->will($this->returnCallback(function () use (&$actualArguments) {
-            $actualArguments = func_get_args();
-        }));
+             ->method('doSomethingElse')
+             ->will(
+                 $this->returnCallback(
+                     function () use (&$actualArguments) {
+                         $actualArguments = func_get_args();
+                     }
+                 )
+             );
 
         $mock->doSomethingElse($expectedObject);
 
@@ -483,7 +606,7 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testObjectMethodCallWithArgumentCloningDisabled()
     {
-        $expectedObject = new StdClass;
+        $expectedObject = new stdClass;
 
         $mock = $this->getMockBuilder('SomeClass')
                      ->setMethods(['doSomethingElse'])
@@ -493,10 +616,14 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
         $actualArguments = [];
 
         $mock->expects($this->any())
-        ->method('doSomethingElse')
-        ->will($this->returnCallback(function () use (&$actualArguments) {
-            $actualArguments = func_get_args();
-        }));
+             ->method('doSomethingElse')
+             ->will(
+                 $this->returnCallback(
+                     function () use (&$actualArguments) {
+                         $actualArguments = func_get_args();
+                     }
+                 )
+             );
 
         $mock->doSomethingElse($expectedObject);
 
@@ -521,11 +648,15 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testVerificationOfMethodNameFailsWithoutParameters()
     {
-        $mock = $this->getMock('SomeClass', ['right', 'wrong'], [], '', true, true, true);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['right', 'wrong'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('right');
 
         $mock->wrong();
+
         try {
             $mock->__phpunit_verify();
             $this->fail('Expected exception');
@@ -542,11 +673,15 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testVerificationOfMethodNameFailsWithParameters()
     {
-        $mock = $this->getMock('SomeClass', ['right', 'wrong'], [], '', true, true, true);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['right', 'wrong'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('right');
 
         $mock->wrong();
+
         try {
             $mock->__phpunit_verify();
             $this->fail('Expected exception');
@@ -563,7 +698,10 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testVerificationOfMethodNameFailsWithWrongParameters()
     {
-        $mock = $this->getMock('SomeClass', ['right', 'wrong'], [], '', true, true, true);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['right', 'wrong'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('right')
              ->with(['first', 'second']);
@@ -604,7 +742,10 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testVerificationOfNeverFailsWithEmptyParameters()
     {
-        $mock = $this->getMock('SomeClass', ['right', 'wrong'], [], '', true, true, true);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['right', 'wrong'])
+                     ->getMock();
+
         $mock->expects($this->never())
              ->method('right')
              ->with();
@@ -624,7 +765,10 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testVerificationOfNeverFailsWithAnyParameters()
     {
-        $mock = $this->getMock('SomeClass', ['right', 'wrong'], [], '', true, true, true);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['right', 'wrong'])
+                     ->getMock();
+
         $mock->expects($this->never())
              ->method('right')
              ->withAnyParameters();
@@ -642,12 +786,12 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
         $this->resetMockObjects();
     }
 
-    /**
-     * @ticket 199
-     */
     public function testWithAnythingInsteadOfWithAnyParameters()
     {
-        $mock = $this->getMock('SomeClass', ['right'], [], '', true, true, true);
+        $mock = $this->getMockBuilder(SomeClass::class)
+                     ->setMethods(['right', 'wrong'])
+                     ->getMock();
+
         $mock->expects($this->once())
              ->method('right')
              ->with($this->anything());
@@ -725,7 +869,7 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
                     ->disableArgumentCloning()
                     ->getMock();
 
-        $a = new stdClass();
+        $a = new stdClass;
         $b = $c = 0;
 
         $foo->expects($this->any())
@@ -747,12 +891,12 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
                     ->disableArgumentCloning()
                     ->getMock();
 
-        $a = new stdClass();
+        $a = new stdClass;
         $b = $c = 0;
 
         $foo->expects($this->any())
             ->method('bar')
-            ->with($this->isInstanceOf('stdClass'), $b, $c)
+            ->with($this->isInstanceOf(stdClass::class), $b, $c)
             ->will($this->returnCallback([$foo, 'callback']));
 
         $foo->bar($a, $b, $c);
@@ -764,6 +908,7 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
     public function testCreateMockFromWsdl()
     {
         $mock = $this->getMockFromWsdl(__DIR__ . '/_fixture/GoogleSearch.wsdl', 'WsdlMock');
+
         $this->assertStringStartsWith(
             'Mock_WsdlMock_',
             get_class($mock)
@@ -776,6 +921,7 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
     public function testCreateNamespacedMockFromWsdl()
     {
         $mock = $this->getMockFromWsdl(__DIR__ . '/_fixture/GoogleSearch.wsdl', 'My\\Space\\WsdlMock');
+
         $this->assertStringStartsWith(
             'Mock_WsdlMock_',
             get_class($mock)
@@ -798,8 +944,8 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
     public function testInterfaceWithStaticMethodCanBeStubbed()
     {
         $this->assertInstanceOf(
-            'InterfaceWithStaticMethod',
-            $this->getMock('InterfaceWithStaticMethod')
+            InterfaceWithStaticMethod::class,
+            $this->getMockBuilder(InterfaceWithStaticMethod::class)->getMock()
         );
     }
 
@@ -808,7 +954,8 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
      */
     public function testInvokingStubbedStaticMethodRaisesException()
     {
-        $mock = $this->getMock('ClassWithStaticMethod');
+        $mock = $this->getMockBuilder(ClassWithStaticMethod::class)->getMock();
+
         $mock->staticMethod();
     }
 
@@ -819,13 +966,21 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
     public function testStubForClassThatImplementsSerializableCanBeCreatedWithoutInvokingTheConstructor()
     {
         $this->assertInstanceOf(
-            'ClassThatImplementsSerializable',
-            $this->getMockBuilder('ClassThatImplementsSerializable')
+            ClassThatImplementsSerializable::class,
+            $this->getMockBuilder(ClassThatImplementsSerializable::class)
                  ->disableOriginalConstructor()
                  ->getMock()
         );
     }
 
+    public function testGetMockForClassWithSelfTypeHint()
+    {
+        $this->assertInstanceOf(
+            ClassWithSelfTypeHint::class,
+            $this->getMockBuilder(ClassWithSelfTypeHint::class)->getMock()
+        );
+    }
+
     private function resetMockObjects()
     {
         $refl = new ReflectionObject($this);
@@ -837,16 +992,29 @@ class Framework_MockObjectTest extends PHPUnit_Framework_TestCase
 
     public function testStringableClassDoesNotThrow()
     {
-        $mock = $this->getMock('StringableClass');
+        $mock = $this->getMockBuilder(StringableClass::class)->getMock();
 
         $this->assertInternalType('string', (string) $mock);
     }
 
     public function testStringableClassCanBeMocked()
     {
-        $mock = $this->getMock('StringableClass');
+        $mock = $this->getMockBuilder(StringableClass::class)->getMock();
+
         $mock->method('__toString')->willReturn('foo');
 
         $this->assertSame('foo', (string) $mock);
     }
+
+    public function traversableProvider()
+    {
+        return [
+          ['Traversable'],
+          ['\Traversable'],
+          ['TraversableMockTestInterface'],
+          [['Traversable']],
+          [['Iterator','Traversable']],
+          [['\Iterator','\Traversable']]
+        ];
+    }
 }
